@@ -6,6 +6,7 @@
 use blib;
 use PDL;
 use PDL::Experiment;
+use PDL::Primitive;
 use PDL::Thread;
 use Data::Dumper;
 
@@ -44,9 +45,22 @@ print $a;
 print "THREADED TRANSPOSE + 4 ->\n";
 print $b;
 
+$a = pdl [[1,2],[3,4],[5,6]],[[0,1],[2,3],[4,5]]
+	;
+$b = byte pdl [[0,0,0],[0,0,0]],[[0,0,0],[0,0,0]]
+	;
+print Dumper(%$b);
+print Dumper(%$a);
+print "B: DATATYPE, $b->{Datatype}\n";
+PDLTEST1($a,$b,4);
+print $a;
+print "THREADED BYTE TRANSPOSE + 4 ->\n";
+print "B: DATATYPE, $b->{Datatype}\n";
+print $b;
+
 print "TESTING CRETION\n";
 
-$b=pdl 0;
+$b=null;
 print $b; 
 $d = {%$b};
 print Dumper($d);
@@ -58,28 +72,8 @@ print Dumper($d);
 
 # Try out the matrix multiplier.
 
-# $c = pdl [0,0,0],[0,0,0],[0,0,0];
-$c = pdl [0,0],[0,0];
-
-PDLTEST2($a,$b,$c);
-
-$a = byte $a;
-$b = short $b;
-$c = byte pdl [0,0],[0,0];
-
-print "DT: $a->{Datatype}, $b->{Datatype} $c->{Datatype}\n";
-
-PDLTEST2($a,$b,$c);
-
-print $a; print $b;
-print $c;
-
-print "DT: $a->{Datatype}, $b->{Datatype} $c->{Datatype}\n";
-
-$a = pdl [0,0.1],[0.2,0.3];
-$b = pdl [0,0],[0,0];
-
-PDLTEST3($a,$b,0);
-
-print $a; print $b;
+$d = null;
+maximum($b,$d);
 
+print "MAXIMUM OF B:\n";
+print $d;
